#if !defined(MODELS_INC)
#define MODELS_INC

class cConstruct;
  class cCylinder;
  class cQuader;
    class cCriss;
    class cCross;

class cConstructModel;

class cEntity;

#include "main.h"

#include "psi3d/cSphere.hpp"
#include "psi3d/cVector.hpp"
#include "psi3d/cMatrix.hpp"
  
#include <list>
#include <string>

using namespace std;


//-----------------------------------


class cConstruct {
  public:
  public:
  enum               { Construct, Cone, ConeStump, Cylinder, Quader, Criss, Cross };
  public:
  char               mParentName[64];
  char               mName[64];
  int                mType;
  bool               mVisible;
  float              mVolume;
  float              mSurface;
  float              mDamage;
  cSphere            mSphere;
//  cPlane             mPlanes[MAX_PLANES];

  cVector            mPos;
  cVector            mRot;

  float*             mRotator3fv;
  float*             mTranslator3fv;

  cMatrix            mMatrix;
  cMatrix            mInvMatrix;

  bool               mTransformed;
  cConstruct        *mParent;
  public:
                     cConstruct();
  virtual           ~cConstruct() {};
  virtual void       Calculate();
  void               ToWorld();
  virtual void       Draw();

  // Checks for line collision.
  // The line a, b will be transformed using mInvMatrix before checking
  // against the local planes.
  // If there is a collision on the line a, b then true is returned.
  // point will contain the collision point.
  // normal will contain the collision normal of the collision point.
  // dist will contain the distance of the collision point to the point a.
  bool collideLine(float* a, float* b, float* point, float* normal, float* dist);
};


class cCylinder : public cConstruct {
  public:
  float              mRadius;
  float              mHeight;
  public:
  void               Calculate();
  void               Draw();
};


class cQuader : public cConstruct {
  public:
  float              mWidth;
  float              mHeight;
  float              mLength;
  public:
  void               Calculate();
  void               Draw();
};


class cCriss : public cQuader {
  public:
  float              mWidth;
  float              mHeight;
  float              mLength;
  public:
  void               Calculate();
  void               Draw();
};


class cCross : public cQuader {
  public:
  float              mWidth;
  float              mHeight;
  float              mLength;
  public:
  void               Calculate();
  void               Draw();
};


//-----------------------------------


class cConstructModel {
  public:
    // model
    string name;
    list<cConstruct*> constructs;
    
    // context
    float matrix[16];
    float invmatrix[16];
    float* rotator3fv;
    float* translator3fv;
    
    list<float*> rotators;
    list<float*> translators;
    list<float*> matrices;
    list<float*> invmatrices;
    
    struct cModelCollision {
        float intsec_distance;
        float intsec_point[3];
        float intsec_normal[3];
    };
    
    
  public:
  
  cConstructModel(char* descriptor);

  bool               createContext(char* descriptor);
  bool               loadSelf(FILE *f);
  
  void               transformSelf();
  void               drawSelf();
  cConstruct*        findConstruct(char* name, list<float*>::iterator &it);

  public:
  void               setRotator(float* rot3fv);
  void               setTranslator(float* tra3fv);

  void               setConstructRotator(char* name, float* rot3fv);
  void               setConstructTranslator(char* name, float* tra3fv);

  float*             getMatrix();
  float*             getInvMatrix();

  float*             getConstructMatrix(char* name);
  float*             getConstructInvMatrix(char* name);

  void               collideLine(float* a, float* b, cModelCollision *&result);
};


#endif
