// cController.hpp
//
// User Control and AI.
// Copyright (c) 1999 by B.Pickhardt (pickhardt@debitel.net).
// aka hackCraft
//

#if !defined(CONTROLLER_INC)
#define CONTROLLER_INC

class cController;
class cObject;

#include <string>
#include <stack>
using namespace std;


class cController {
  struct rInstruction {
    int opcode;
    string* entity_name;
    cObject* entity;
  };
  
public:
  enum Opcodes { WAIT, ENGAGE, TARGET, ATTACK, FOLLOW, FLEE, ROUTE, PATROL };
  const char* getInstructionName(int opcode) {
      const char* names[] = { "WAIT", "ENGAGE", "TARGET", "ATTACK", "FOLLOW", "FLEE", "ROUTE", "PATROL" };
      return names[opcode];
  };
  
  public:
  cObject* mDevice; // Controlled entity cObject.
  bool mEnabled; // Whether controll is enabled (autopilot/ai enabled/disabled).

public:
  stack<rInstruction*> mInstructions;
public:
  cController(cObject* entity, bool enabled = true);
  virtual ~cController() {};

  void processSelf();

  void clearInstruction();
  void overideInstruction(int opcode, cObject* entity);
  void overideInstruction(int opcode, char* entityname);
  void newInstruction(int opcode, cObject* entity);
  void newInstruction(int opcode, char* entityname);
  void oldInstruction();

  // Transition Functions
  void waitEnemy();
  void engageEnemy();
  void targetEnemy();
  void attackEnemy();
  void followLeader();
  void fleeEnemy();
  void routeNav();
  void patrolArea();
  
  // Queries
  cObject* enemyNearby();
  
  // Fuzzy Predicates [0,1]
  float inMeeleRange();
  float inWeaponRange();
  float inTargetRange();
  
  // Output Actions
  void do_moveTowards();
  void do_moveNear();
  void do_aimAt();
  void do_fireAt();
  void do_idle();
  void do_aimFor(cObject* target);
  
};


#endif
