// cMission.h

#ifndef _CMISSION_H
#define _CMISSION_H

class cMission;

#include "cWorld.h"
#include "cObject.h"
#include "cPad.h"

/**
 * Missions are responsible for initial and midtime spawning of objects
 * (including the player object), tracking mission objectives (frags).
 * 
 * Future: support for timetriggers like onTime().
 */
class cMission {
  public:
  
  /**
   * Called with the world which needs to be populated by this mission.
   * So this method should spawn objects into the world and setup
   * other mission critical data of the derived mission.
   */
  virtual cObject* init(cWorld* world) = 0;
  
  /**
   * Notifies that the given object was fragged to the derived mission
   * controller. Through this method mission objectives can be tracked.
   */
  virtual void onFrag(cObject* fragged) = 0;
  
  /**
   * Indicates wether this Mission is a success.
   */
  //virtual bool isSuccess() = 0;
  
  /**
   * Indicates wether this Mission is a failure.
   */
  //virtual bool isFailure() = 0;
};


class cTestMission : public cMission {
  cObject* player;
  cObject* primary;
  public:
  virtual cObject* init(cWorld* world);
  virtual void onFrag(cObject* fragged);
  private:
  cObject* initMechs(cWorld* world);
  void initFlora(cWorld* world);
  void initArchitecture(cWorld* world);
};

#endif

