// cPad.hpp
//
// Provides Uniform GamePad-Input in a Common Fashion.
// Possibly-Mapped-Input-Sources:
//   Keyboard(+Mouse), Real-GamePad, AI-Controll, Remote-Controll (&Proxies).
// Not yet: Input recording and playback for demoloops.

#if !defined(PAD_INC)
#define PAD_INC

#include <string>
using namespace std;

class cPad {
  private:
  char axes[4];
  unsigned short buttons;
  public:
  enum Axes { AX_UD1, AX_LR1, AX_UD2, AX_LR2 };
  enum Buttons { BT_HU, BT_HD, BT_HL, BT_HR, BT_DELTA, BT_CROSS, BT_SQUARE, BT_CIRCLE, BT_L1, BT_R1, BT_L2, BT_R2, BT_START, BT_SELECT, BT_J1B, BT_J2B };
  cPad() { setZero(); };
  void setZero() { axes[0]=axes[1]=axes[2]=axes[3]=0; buttons=0; };
  float getAxis(Axes axis) { return (((float)this->axes[axis]) / 127.0f); };
  int getButton(Buttons button) { if ( (this->buttons & (1UL<<button)) != 0) return 1; return 0; };
  void setAxis(Axes axis, int value) { this->axes[axis] = (value < -127) ? -127 : (value > +127) ? +127 : value; };
  void setAxis(Axes axis, float value) { this->axes[axis] = (char) ( 127.0f*((value < -1.0f) ? -1.0f : (value > +1.0f) ? +1.0f : value) ); };
  void setButton(Buttons button, bool enable) { if (enable) this->buttons |= (1UL<<button); else this->buttons &= ~(1UL<<button); };
  void print() {
    printf("Pad: J1(%1.2f, %1.2f) J2(%1.2f, %1.2f) HUD(%i, %i, %i, %i) ABXY(%i, %i, %i, %i) L1/R1(%i, %i) L2/R2(%i, %i) SEL/ST(%i, %i) J1B(%i) J2B(%i)\n",
      getAxis(AX_UD1), getAxis(AX_LR1),
      getAxis(AX_UD2), getAxis(AX_LR2),
      getButton(BT_HU), getButton(BT_HD), getButton(BT_HL), getButton(BT_HR), 
      getButton(BT_DELTA),  getButton(BT_CROSS),  getButton(BT_SQUARE),  getButton(BT_CIRCLE),
      getButton(BT_L1),  getButton(BT_R1),
      getButton(BT_L2),  getButton(BT_R2),
      getButton(BT_SELECT),  getButton(BT_START),
      getButton(BT_J1B),  getButton(BT_J2B)
      );
  };
  char* serialise(char *buffer) {
    int size = 8;
    memcpy(buffer, this, size);
    return (buffer + size);
  }
  char* deserialise(char* buffer) {
    int size = 8;
    memcpy(this, buffer, size);
    return (buffer + size);
  }
};

#endif
