/* 
 * File:   macros.h
 * Author: Benjamin Pickhardt (hackCraft)
 *
 * Created on March 11, 2008, 3:04 PM
 */

#ifndef _MACROS_H
#define	_MACROS_H

// Controll Structures


#define looplist(loopvar, itemtype, listvar) \
for (list<itemtype>::iterator loopvar = (listvar).begin(); loopvar != (listvar).end(); loopvar++)


// Misc.


#define NATO_Alphabet_Array(name) \
char* name[] = \
{ \
    "Alpha", "Bravo", "Charlie", "Delta", "Echo", "Foxtrot", "Golf", "Hotel", "India", "Juliett", "Kilo", "Lima", "Mike", "November", "Oscar", "Papa", \
    "Quibec", "Romeo", "Sierra", "Tango", "Uniform", "Victor", "Whiskey", "X-Ray", "Yankee", "Zulu" \
};

#define getSystemHour(hour1f) \
{ \
  time_t t = time(NULL); \
  tm *l = localtime(&t); \
  hour1f = (float) (l->tm_hour + l->tm_min / 60.0 + l->tm_sec / 3600.0); \
}


// Primitives


#define glVertex3fv2f(pnt, u, v) \
{ \
    glTexCoord2f(u,v); \
    glVertex3fv(pnt); \
}


#define glNormAxis() \
{ \
    glPushAttrib(GL_CURRENT_BIT | GL_ENABLE_BIT); \
    glDisable(GL_TEXTURE_2D); \
    glDisable(GL_LIGHTING); \
    glBegin(GL_LINES); \
        glColor4f(1,0,0,1); \
        glVertex3f(0,0,0); \
        glVertex3f(1,0,0); \
        glColor4f(0,1,0,1); \
        glVertex3f(0,0,0); \
        glVertex3f(0,1,0); \
        glColor4f(0,0,1,1); \
        glVertex3f(0,0,0); \
        glVertex3f(0,0,1); \
    glEnd(); \
    glPopAttrib(); \
}


#define glDisk(segments, radius) \
{ \
    int n = 2 + (segments); \
    float step = 2*M_PI / (float) n; \
    float a = 0; \
    glBegin(GL_TRIANGLE_FAN); \
    glNormal3f(0,0,1); \
    for (int i = 0; i < n; i++) { \
        glVertex3f(sin(a)*(radius), cos(a)*(radius), 0); \
        a += step; \
    } \
    glEnd(); \
}


#define glUnitCylinder(segments) \
{ \
    int n = 2 + (segments); \
    float step = 1.0f / (float) n; \
    float anglestep = 2.0f * M_PI *step; \
    float texture = 0.0f; \
    float angle = 0.0f; \
    glBegin(GL_TRIANGLE_STRIP); \
    for (int i=0; i<=n; i++) { \
        glTexCoord2f(texture, +1.0f); \
        glVertex3f(sin(angle), +1.0f, cos(angle)); \
        glTexCoord2f(texture, 0); \
        glVertex3f(sin(angle), +0.0f, cos(angle)); \
        texture += step; \
        angle += anglestep; \
    }; \
    glEnd(); \
}


#define glCenterUnitCylinder(segments) \
{ \
    float step = 1.0f / (float) segments; \
    float anglestep = 2.0f * M_PI *step; \
    float texture = 0.0f; \
    float angle = 0.0f; \
    glBegin(GL_TRIANGLE_STRIP); \
    for (int i=0; i<=segments; i++) { \
        glTexCoord2f(texture, +1.0f); \
        glVertex3f(sin(angle), +1.0f, cos(angle)); \
        glTexCoord2f(texture, 0); \
        glVertex3f(sin(angle), -1.0f, cos(angle)); \
        texture += step; \
        angle += anglestep; \
    }; \
    glEnd(); \
}


#define glUnitBlock() \
{ \
    GLfloat p[][3] = { \
        {0,0,0}, {1,0,0}, {1,1,0}, {0,1,0}, \
        {0,0,1}, {1,0,1}, {1,1,1}, {0,1,1} \
    }; \
     \
    glBegin(GL_QUADS); \
        /* front */ \
        glNormal3f(0, 0, -1); \
        glVertex3fv2f(p[1], 1,0); glVertex3fv2f(p[0], 0,0); \
        glVertex3fv2f(p[3], 0,1); glVertex3fv2f(p[2], 1,1); \
        /* back */ \
        glNormal3f(0, 0, +1); \
        glVertex3fv2f(p[4], 0,0); glVertex3fv2f(p[5], 1,0); \
        glVertex3fv2f(p[6], 1,1); glVertex3fv2f(p[7], 0,1); \
        /* top */ \
        glNormal3f(0, -1, 0); \
        glVertex3fv2f(p[0], 0,0); glVertex3fv2f(p[1], 1,0); \
        glVertex3fv2f(p[5], 1,0); glVertex3fv2f(p[4], 0,0); \
        /* bottom */ \
        glNormal3f(0, +1, 0); \
        glVertex3fv2f(p[2], 1,1); glVertex3fv2f(p[3], 0,1); \
        glVertex3fv2f(p[7], 0,0); glVertex3fv2f(p[6], 1,0); \
        /* left */ \
        glNormal3f(+1, 0, 0); \
        glVertex3fv2f(p[0], 0,0); glVertex3fv2f(p[4], 1,0); \
        glVertex3fv2f(p[7], 1,1); glVertex3fv2f(p[3], 0,1); \
        /* right */ \
        glNormal3f(-1, 0, 0); \
        glVertex3fv2f(p[5], 0,0); glVertex3fv2f(p[1], 1,0); \
        glVertex3fv2f(p[2], 1,1); glVertex3fv2f(p[6], 0,1); \
    glEnd(); \
}


#define glCenterUnitBlock() \
{ \
    GLfloat p[][3] = { \
        {-1,-1,-1}, {+1,-1,-1}, {+1,+1,-1}, {-1,+1,-1}, \
        {-1,-1,+1}, {+1,-1,+1}, {+1,+1,+1}, {-1,+1,+1} \
    }; \
     \
    glBegin(GL_QUADS); \
        /* front */ \
        glNormal3f(0, 0, -1); \
        glVertex3fv2f(p[1], 1,0); glVertex3fv2f(p[0], 0,0); \
        glVertex3fv2f(p[3], 0,1); glVertex3fv2f(p[2], 1,1); \
        /* back */ \
        glNormal3f(0, 0, +1); \
        glVertex3fv2f(p[4], 0,0); glVertex3fv2f(p[5], 1,0); \
        glVertex3fv2f(p[6], 1,1); glVertex3fv2f(p[7], 0,1); \
        /* top */ \
        glNormal3f(0, -1, 0); \
        glVertex3fv2f(p[0], 0,0); glVertex3fv2f(p[1], 1,0); \
        glVertex3fv2f(p[5], 1,0); glVertex3fv2f(p[4], 0,0); \
        /* bottom */ \
        glNormal3f(0, +1, 0); \
        glVertex3fv2f(p[2], 1,1); glVertex3fv2f(p[3], 0,1); \
        glVertex3fv2f(p[7], 0,0); glVertex3fv2f(p[6], 1,0); \
        /* left */ \
        glNormal3f(+1, 0, 0); \
        glVertex3fv2f(p[0], 0,0); glVertex3fv2f(p[4], 1,0); \
        glVertex3fv2f(p[7], 1,1); glVertex3fv2f(p[3], 0,1); \
        /* right */ \
        glNormal3f(-1, 0, 0); \
        glVertex3fv2f(p[5], 0,0); glVertex3fv2f(p[1], 1,0); \
        glVertex3fv2f(p[2], 1,1); glVertex3fv2f(p[6], 0,1); \
    glEnd(); \
}


// Pushing and Poping different Projection-Matrices:
// Note that the current Matrix-Mode is saved and
// stays the same after calling the projection-macros (most often MODEL_VIEW).

#define glPushPerspectiveProjection(fovdegrees) \
{ \
    glPushAttrib(GL_DEPTH_BUFFER_BIT); /* push glDepthFunc */ \
    glPushAttrib(GL_TRANSFORM_BIT); \
        glMatrixMode(GL_PROJECTION); \
        glPushMatrix(); \
        glLoadIdentity(); \
        GLint view[4]; \
        glGetIntegerv(GL_VIEWPORT, view); \
        gluPerspective(fovdegrees, GLdouble(view[2]) / GLdouble(view[3]), 0.05, 5000.0); \
        glDepthFunc(GL_LEQUAL); \
    glPopAttrib(); \
}


#define glPushUnitOrthoProjection() \
{ \
    glPushAttrib(GL_DEPTH_BUFFER_BIT); /* push glDepthFunc */ \
    glPushAttrib(GL_TRANSFORM_BIT); \
        glMatrixMode(GL_PROJECTION); \
        glPushMatrix(); \
        glLoadIdentity(); \
        glOrtho(0, 1, 0, 1, -5000.0, 5000.0); \
        glDepthFunc(GL_GEQUAL); \
    glPopAttrib(); \
}


#define glPushCenterUnitOrthoProjection() \
{ \
    glPushAttrib(GL_DEPTH_BUFFER_BIT); /* push glDepthFunc */ \
    glPushAttrib(GL_TRANSFORM_BIT); \
        glMatrixMode(GL_PROJECTION); \
        glPushMatrix(); \
        glLoadIdentity(); \
        glOrtho(-1, 1, -1, 1, -5000.0, 5000.0); \
        glDepthFunc(GL_GEQUAL); \
    glPopAttrib(); \
}


#define glPushViewportOrthoProjection() \
{ \
    glPushAttrib(GL_DEPTH_BUFFER_BIT); /* push glDepthFunc */ \
    glPushAttrib(GL_TRANSFORM_BIT); \
        glMatrixMode(GL_PROJECTION); \
        glPushMatrix(); \
        glLoadIdentity(); \
        GLint view[4]; \
        glGetIntegerv(GL_VIEWPORT, view); \
        glOrtho(view[0], view[2], view[1], view[3], -5000.0, 5000.0); \
        glDepthFunc(GL_GEQUAL); \
    glPopAttrib(); \
}


#define glPopProjection() \
{ \
    glPushAttrib(GL_TRANSFORM_BIT); \
        glMatrixMode(GL_PROJECTION); \
        glPopMatrix(); \
    glPopAttrib(); \
    glPopAttrib(); /* pop glDepthFunc */ \
}


// Billboarding
// Extracts and inverts the rotational part of the MODEL_VIEW_MATRIX
// by embedding the transposed upper 3x3 into an otherwise
// identity 4x4 matrix - assumes that no scaling was applied.
// The result can be glMultMatrix'ed in to eliminate rotations
// for billboarding.

#define glGetTransposeInverseRotationMatrix(invm) \
{ \
    float _m_[16]; \
    glGetFloatv(GL_MODELVIEW_MATRIX, _m_); \
    float _n_[] = { /* = m Transposed = m Inverted */  \
        _m_[0], _m_[4], _m_[ 8],  0, \
        _m_[1], _m_[5], _m_[ 9],  0, \
        _m_[2], _m_[6], _m_[10],  0, \
             0,      0,       0,  1 \
    }; \
    memcpy(invm, _n_, sizeof(float)*16); \
}


#endif	/* _MACROS_H */

